module Character exposing (..)

import Creation exposing (CreationPoints)
import Stats exposing (Stat)
import Stats exposing (AttributeCategory)

type alias Character =
    { name : String
    , creationPoints : CreationPoints
    , stats : List Stat
    }

new : Character
new =
    { name = "Default Name"
    , creationPoints = Creation.new
    , stats = Stats.new
    }

changeName : Character -> String -> Character
changeName character newName =
    if ( newName == "")
    then
        character
    else
        { character | name = newName }

changeStat : Character -> Stat -> Character
changeStat character stat =
    case stat.statType of
        Stats.Arete -> character
        Stats.Sphere ->
            let
                spheres = Stats.spheresInList character.stats
                arete = Stats.areteInList character.stats

                maybeOldDots = Stats.dotsOfStatInList spheres stat.name
                difference = case maybeOldDots of
                    Nothing -> 0
                    Just oldDots ->
                        if stat.dots.value > arete.dots.value
                        then arete.dots.value - oldDots.value
                        else stat.dots.value  - oldDots.value
            in
                case maybeOldDots of
                    Nothing -> character
                    Just oldDots ->
                        { character
                        | creationPoints = Creation.buySphere character.creationPoints difference
                        , stats = Stats.changeStatInList character.stats ( Stats.changeDotsTo stat ( oldDots.value + difference ) )
                        }
        Stats.Attribute _ -> 
            character

changePriority : Character -> AttributeCategory -> Int -> Character
changePriority character category value =
    { character
    | creationPoints = Creation.changePriority character.creationPoints category value 
    }