module Stats exposing (..)

import BoundedInt exposing (BoundedInt)

type alias Stat =
    { name : String
    , dots : BoundedInt
    , statType : StatType
    }

type StatType
    = Sphere
    | Arete
    | Attribute AttributeCategory

type AttributeCategory
    = Physical
    | Social
    | Mental

new : List Stat
new =
    [ newSphere "Correspondence"
    , newSphere "Entropy"
    , newSphere "Forces"
    , newSphere "Life"
    , newSphere "Matter"
    , newSphere "Mind"
    , newSphere "Prime"
    , newSphere "Spirit"
    , newSphere "Time"
    , newArete
    , newAttribute Physical "Strength"
    , newAttribute Physical "Dexterity"
    , newAttribute Physical "Stamina"
    , newAttribute Social "Charisma"
    , newAttribute Social "Manipulation"
    , newAttribute Social "Appearance"
    , newAttribute Mental "Perception"
    , newAttribute Mental "Intelligence"
    , newAttribute Mental "Wits"
    ]

newSphere : String -> Stat
newSphere name =
    { name = name
    , dots = BoundedInt.new 0 0 5
    , statType = Sphere
    }

newArete : Stat
newArete =
    { name = "Arete"
    , dots = BoundedInt.new 1 1 10
    , statType = Arete
    }

newAttribute : AttributeCategory -> String -> Stat
newAttribute category name =
    { name = name
    , dots = BoundedInt.new 1 1 5
    , statType = Attribute category
    }

spheresInList : List Stat -> List Stat
spheresInList stats =
    List.filter isSphere stats

isSphere : Stat -> Bool
isSphere stat =
    stat.statType == Sphere

areteInList : List Stat -> Stat
areteInList stats =
    let 
        maybeArete = List.head ( List.filter isArete stats )
    in
        case maybeArete of
            Just arete -> arete
            Nothing -> newArete

isArete : Stat -> Bool
isArete stat =
    stat.statType == Arete

attributesInList : List Stat -> List Stat
attributesInList stats =
    List.filter isAttribute stats

isAttribute : Stat -> Bool
isAttribute stat =
    case stat.statType of
        Attribute _ -> True
        _ -> False

attributesOfCategoryInList : AttributeCategory -> List Stat -> List Stat
attributesOfCategoryInList category stats =
    List.filter ( isAttributeOfCategory category ) stats

isAttributeOfCategory : AttributeCategory -> Stat -> Bool
isAttributeOfCategory category stat =
    stat.statType == Attribute category

dotsOfStatInList : List Stat -> String -> Maybe BoundedInt
dotsOfStatInList stats name =
    let
        maybeStat = stats
            |> List.filter ( isStat name )
            |> List.head
    in
        case maybeStat of
            Nothing -> Nothing
            Just stat ->
                Just stat.dots

isStat : String -> Stat -> Bool
isStat name stat =
    stat.name == name

changeStatInList : List Stat -> Stat -> List Stat
changeStatInList stats stat =
    List.map ( changeStatIfSame stat ) stats

changeStatIfSame : Stat -> Stat -> Stat
changeStatIfSame newStat oldStat =
    if isStat newStat.name oldStat
    then
        newStat
    else
        oldStat

changeDotsTo : Stat -> Int -> Stat
changeDotsTo stat value =
    { stat | dots = BoundedInt.changeTo stat.dots value }

everyThirdStat : List Stat -> Int -> List Stat
everyThirdStat stats offset =
    stats
    |> List.indexedMap Tuple.pair
    |> List.filter ( \x -> ( Tuple.first x ) // 3 == offset )
    |> List.map Tuple.second